#!/usr/bin/python3.7+
# -*- coding:utf-8 -*-
"""
@auth: lxm
@date: 2020/12/14
@desc:
fastapi请求上下文功能
参考
https://github.com/tomwojcik/starlette-context
https://docs.python.org/zh-cn/3.7/library/contextvars.html#module-contextvars
"""
from .fastapi_context import context
from .context_middleware import ContextMiddleware



#!/usr/bin/python3.6+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-9-13
@desc: ...
"""
import os

try:
    import yaml
except:
    yaml = None

from pydantic import BaseSettings, AnyUrl
from yzcore.utils import get_random_secret_key


class DefaultSetting(BaseSettings):
    __cml__ = {}

    def __init_subclass__(cls, **kwargs):
        """"""
        super().__init_subclass__()
        reload_reload_settings(cls())

    class Config:
        case_sensitive = False  # 是否区分大小写

    DEBUG: bool = True
    API_V1_STR: str = "/api/v1"
    SECRET_KEY: str = get_random_secret_key()
    BASE_URL: str = None

    DB_URI: str = None
    ID_URL: AnyUrl = None
    GENERATE_UUID_PATH: str = '/uuid/generate/'
    EXPLAIN_UUID_PATH: str = '/uuid/explain/'
    TRANSLATE_PATH: str = '/uuid/translate/'
    MAKE_UUID_PATH: str = '/uuid/make/'

    STORAGE_CONF: dict = None


default_setting = DefaultSetting()


def reload_reload_settings(instance):
    settings = default_setting
    for k, v in settings.__fields__.items():
        val = getattr(instance, k)
        setattr(settings, k, val)


def get_configer(ext: str = "ini", import_path=os.curdir):
    profile = os.environ.get('ENV_PROFILE', 'dev')
    if profile == 'production':
        configname = 'config_production'
    elif profile == 'testing':
        configname = 'config_testing'
    else:
        configname = 'config_dev'
    print(f"===>当前环境为：{profile}!导入的配置文件为：{configname}.{ext}")

    base_path = os.path.abspath(import_path)
    _path = os.path.join(base_path, "conf", f"{configname}.{ext}")
    print(_path)
    if ext in ["ini", "cfg"]:
        import configparser
        conf = configparser.ConfigParser()
        conf.read(_path)
    elif ext in ["yaml", "yml"]:
        assert yaml is not None, "Need to install PyYaml"
        conf = yaml.safe_load(open(_path))
    else:
        raise AttributeError(f"暂不支持该文件格式: {ext}")
    return conf


def get_ini_section_to_dict(
        section: str,
        exclude: set = None,
        conf_parser=None
):
    """
    获取ini配置文件某个节选的全部数据，转换成字典

    :param section: 节选名称
    :param exclude: 排除的字段
    :param conf_parser: 配置解析器
    :return:
    """
    conf_dict = dict()
    for k in conf_parser.options(section):
        if exclude and k in exclude:
            break
        conf_dict[k] = conf.get(section, k)
    return conf_dict


if __name__ == '__main__':
    conf = get_configer("ini")




try:
    import oss2

    ACL_TYPE = {
        "private": oss2.BUCKET_ACL_PRIVATE,
        "onlyread": oss2.BUCKET_ACL_PUBLIC_READ,
        "readwrite": oss2.BUCKET_ACL_PUBLIC_READ_WRITE,
    }
    # 存储类型
    STORAGE_CLS = {
        "standard": oss2.BUCKET_STORAGE_CLASS_STANDARD,  # 标准类型
        "ia": oss2.BUCKET_STORAGE_CLASS_IA,  # 低频访问类型
        "archive": oss2.BUCKET_STORAGE_CLASS_ARCHIVE,  # 归档类型
        "cold_archive": oss2.BUCKET_STORAGE_CLASS_COLD_ARCHIVE,  # 冷归档类型
    }
    # 冗余类型
    REDUNDANCY_TYPE = {
        "lrs": oss2.BUCKET_DATA_REDUNDANCY_TYPE_LRS,  # 本地冗余
        "zrs": oss2.BUCKET_DATA_REDUNDANCY_TYPE_ZRS,  # 同城冗余（跨机房）
    }

except ImportError:
    pass



#!/usr/bin/python3.7+
# -*- coding:utf-8 -*-
"""
@auth: xxx
@date: 2020-9-
@desc: ...
"""
from sqlalchemy import text
from sqlalchemy.orm import Session
from yzcore.db.sqlalchemy_crud_base import OrmCRUDBase
from .models import Permissions


class PermissionController(OrmCRUDBase):
    def __init__(self):
        super().__init__(Permissions)

    def get_permission(
            self, db: Session,
            obj_id: int, obj_type: int,
            organiz_id: int, user_id: int
    ):
        """
        获取权限

        :param db:
        :param obj_id:
        :param obj_type:
        :param organiz_id:
        :param user_id:
        :return:
        """
        query = dict(
            object_id=obj_id,
            object_type=obj_type,
            organiz_id=organiz_id,
            user_id=user_id
        )
        pm_obj = self.get_one(db, **query)
        if pm_obj:
            return pm_obj.permission
        return 0


#!/usr/bin/python3.7+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-9-
@desc: ...
"""
from fastapi import APIRouter,HTTPException
from fastapi import Query,Body
from yzcore.response import response
from yzcore.logger import get_logger
from yzcore.exceptions import CreateObjectFailed, UpdateObjectFailed
from .schemas import PermissionBase


logger = get_logger(__name__)
logger.debug('test_debug')


router = APIRouter()


@router.get('/resources/{resource_id}')
def view_get_info(resource_id: int):
    """
    GET 示例

        :param resource_id:
        :return:
    """
    raise HTTPException(status_code=404, detail='找不到对象')


@router.get('/resources/')
def view_list_resources(
        query: str = Query('keyword'),
        limit: int = Query(10, gt=0, le=1000),
        offset: int = Query(0, ge=0),
):
    """
    GET 列表查询示例

        :param query:
        :param limit:
        :param offset:
        :return:
    """


@router.post("/resources/")
def view_create_resources(
        body: PermissionBase
):
    """
    POST 示例

        :param body:
        :return:
    """
    raise CreateObjectFailed()


@router.put("/resources/{resource_id}")
def view_update_resources(
        resource_id: int,
        body: PermissionBase
):
    """
    PUT 全量更新示例

        :param body:
        :return:
    """
    raise UpdateObjectFailed()


@router.patch("/resources/{resource_id}")
def view_update_resources(
        resource_id: int,
        body: PermissionBase
):
    """
    PATCH 局部更新示例

        :param body:
        :return:
    """
    raise UpdateObjectFailed()


#!/usr/bin/env python3.5+
# -*- coding: utf-8 -*-
"""
@auth: cml
@date: 2019-07-10
@desc: ...
"""
import logging


class NotSetFilter(logging.Filter):

    def filter(self, record):
        return False


class DebugFilter(logging.Filter):

    def filter(self, record):
        level = record.levelname.upper()
        if level == 'DEBUG':
            return True
        return False


class InfoFilter(logging.Filter):

    def filter(self, record):
        level = record.levelname.upper()
        # if level in {'INFO', 'WARNING', 'ERROR'}:
        if level == 'INFO':
            return True
        return False


class WarningFilter(logging.Filter):

    def filter(self, record):
        level = record.levelname.upper()
        if level == 'WARNING':
            return True
        return False


class ErrorFilter(logging.Filter):

    def filter(self, record):
        level = record.levelname.upper()
        if level == 'ERROR':
            return True
        return False


class CriticalFilter(logging.Filter):

    def filter(self, record):
        level = record.levelname.upper()
        if level == 'CRITICAL':
            return True
        return False


#!/usr/bin/python3.6.8+
# -*- coding:utf-8 -*-
"""
@auth: xxx
@date: 2020-10-18
@desc: ...
"""
from pydantic import BaseModel, validator, ValidationError, Field
from typing import Type, Optional, Any, List, Tuple
from yzcore import exceptions as exp


class CommonBase(BaseModel):
    class Config:
        orm_mode = True


class PermissionBase(CommonBase):
    object_id: int
    object_type: int
    parent_id: Optional[int]
    parent_type: Optional[int]

    @validator('object_type')
    def validate_object_type(cls, value, values):
        """object_type in [1,2,3,4]"""
        if value not in [1, 2]:
            raise exp.RequestParamsError('该类型不能创建权限')

        # 为后期权限颗粒度变细准备：
        if value > 2 and not all((values['parent_id'], values['parent_type'])):
            raise exp.RequestParamsError('该类型需要传入parent')
        return value



#!/usr/bin/python3.6+
# -*- coding:utf-8 -*-
"""
@author: zhouwei
@date: 2022/11/09
@desc: minio对象存储封装
"""
import json
import traceback
from datetime import timedelta, datetime
from os import PathLike
from typing import Union, IO, AnyStr

from yzcore.extensions.storage.base import StorageManagerBase, StorageRequestError, logger
from yzcore.extensions.storage.schemas import MinioConfig
from yzcore.extensions.storage.utils import AnyStr2BytesIO
from yzcore.extensions.storage.minio.utils import wrap_request_return_bool, wrap_request_raise_404
from yzcore.utils.time_utils import datetime2str


try:
    from minio import Minio
    from minio.datatypes import PostPolicy
    from minio.commonconfig import CopySource
    from minio.deleteobjects import DeleteObject
    from minio.error import S3Error
except:
    Minio = None


class MinioManager(StorageManagerBase):

    def __init__(self, conf: MinioConfig):
        super(MinioManager, self).__init__(conf)
        self.internal_endpoint = conf.internal_endpoint
        self.disable_internal_endpoint = conf.disable_internal_endpoint
        self.internal_minioClient = None

        self.__init()

    def __init(self, bucket_name=None):
        """初始化对象"""

        if Minio is None:
            raise ImportError("'minio' must be installed to use MinioManager")

        if bucket_name:
            self.bucket_name = bucket_name

        self.minioClient = Minio(
            self.endpoint,
            access_key=self.access_key_id,
            secret_key=self.access_key_secret,
            secure=True if self.scheme == 'https' else False,
        )

        if self.internal_endpoint and not self.disable_internal_endpoint:
            self.internal_minioClient = Minio(
                self.internal_endpoint,
                access_key=self.access_key_id,
                secret_key=self.access_key_secret,
                secure=False,
            )

    def _internal_minio_client_first(self):
        """优先使用内网连接minio服务"""
        if self.internal_minioClient:
            return self.internal_minioClient
        else:
            return self.minioClient

    def get_bucket_cors(self):
        """返回的内容格式和OSS/OBS差异太大"""
        client = self._internal_minio_client_first()
        result = client.get_bucket_policy(self.bucket_name)
        return json.loads(result)

    def set_bucket_cors(self, policy: dict):
        client = self._internal_minio_client_first()
        return client.set_bucket_policy(self.bucket_name, policy)

    def _cors_check(self):
        passed = False
        action_slots = ['s3:AbortMultipartUpload', 's3:DeleteObject', 's3:GetObject',
                        's3:ListMultipartUploadParts', 's3:PutObject']

        cors_dict = self.get_bucket_cors()
        logger.debug(f'_cors_check: {cors_dict}')
        for cors in cors_dict['Statement']:
            effect = cors['Effect']
            resource = cors['Resource'][0]
            actions = cors['Action']
            if effect == 'Allow':
                if resource == f'arn:aws:s3:::{self.bucket_name}/*' or resource == 'arn:aws:s3:::*':
                    if all([False for i in action_slots if i not in actions]):
                        passed = True
        return passed

    def create_bucket(self, bucket_name=None):
        """创建bucket，并且作为当前操作bucket"""
        client = self._internal_minio_client_first()
        client.make_bucket(bucket_name)
        self.bucket_name = bucket_name

    def list_buckets(self):
        client = self._internal_minio_client_first()
        return client.list_buckets()

    def is_exist_bucket(self, bucket_name=None):
        client = self._internal_minio_client_first()
        if bucket_name is None:
            bucket_name = self.bucket_name
        return client.bucket_exists(bucket_name)

    def delete_bucket(self, bucket_name=None):
        client = self._internal_minio_client_first()
        if bucket_name is None:
            bucket_name = self.bucket_name
        return client.remove_bucket(bucket_name)

    def get_sign_url(self, key, expire=0):
        expire_time = timedelta(seconds=expire or self.private_expire_time)
        url = self.minioClient.presigned_get_object(self.bucket_name, key, expires=expire_time)
        return '//' + url.split('//', 1)[-1]

    def post_sign_url(self, key):
        client = self._internal_minio_client_first()
        expire_time = datetime.now() + timedelta(seconds=self.policy_expire_time)
        policy = PostPolicy(bucket_name=self.bucket_name, expiration=expire_time)
        policy.add_starts_with_condition('$key', key)
        # policy.add_content_length_range_condition(1, 1024*1024*1024)  # 限制文件大小
        return client.presigned_post_policy(policy)

    def put_sign_url(self, key):
        return self.minioClient.presigned_put_object(self.bucket_name, key)

    def iter_objects(self, prefix='', **kwargs):
        client = self._internal_minio_client_first()
        objects = client.list_objects(self.bucket_name, prefix=prefix)
        _result = []
        for obj in objects:
            _result.append({
                'key': obj.object_name,
                'url': self.get_file_url(obj.object_name),
                'size': obj.size,
            })
        return _result

    @wrap_request_raise_404
    def get_object_meta(self, key: str):
        """获取文件基本元信息，包括该Object的ETag、Size（文件大小）、LastModified，Content-Type，并不返回其内容"""
        client = self._internal_minio_client_first()
        meta = client.stat_object(self.bucket_name, key)
        return {
            'etag': meta.etag,
            'size': meta.size,
            'last_modified': datetime2str(meta.last_modified),
            'content_type': meta.content_type,
        }

    @wrap_request_raise_404
    def _set_object_headers(self, key: str, headers: dict):
        """更新文件的metadata，主要用于更新Content-Type"""
        client = self._internal_minio_client_first()
        client.copy_object(self.bucket_name, key, CopySource(self.bucket_name, key), metadata=headers, metadata_directive='REPLACE')
        return True

    @wrap_request_return_bool
    def file_exists(self, key):
        client = self._internal_minio_client_first()
        return client.stat_object(self.bucket_name, key)

    @wrap_request_raise_404
    def download_stream(self, key, **kwargs):
        client = self._internal_minio_client_first()
        return client.get_object(self.bucket_name, key)

    @wrap_request_raise_404
    def download_file(self, key, local_name, **kwargs):
        client = self._internal_minio_client_first()
        client.fget_object(self.bucket_name, key, local_name)

    def upload_file(self, filepath: Union[str, PathLike], key: str, **kwargs):
        """上传文件"""
        client = self._internal_minio_client_first()
        try:
            content_type = self.parse_content_type(key)
            client.fput_object(self.bucket_name, key, filepath, content_type=content_type)
            return self.get_file_url(key)
        except Exception:
            logger.error(f'minio upload error: {traceback.format_exc()}')
            raise StorageRequestError('minio upload error')

    def upload_obj(self, file_obj: Union[IO, AnyStr], key: str, **kwargs):
        """上传文件流"""
        client = self._internal_minio_client_first()
        try:
            if isinstance(file_obj, (str, bytes)):
                file_obj = AnyStr2BytesIO(file_obj)
            content_type = self.parse_content_type(key)
            client.put_object(self.bucket_name, key, file_obj, length=-1, content_type=content_type,
                              part_size=1024 * 1024 * 5)
            return self.get_file_url(key)
        except Exception:
            logger.error(f'minio upload error: {traceback.format_exc()}')
            raise StorageRequestError('minio upload error')

    def delete_object(self, key: str):
        """删除文件"""
        client = self._internal_minio_client_first()
        errors = client.remove_objects(self.bucket_name, [DeleteObject(key)])
        for error in errors:
            logger.error(f'minio delete file error: {error}')
            raise StorageRequestError('minio delete file error')
        return True

    def get_policy(
            self,
            filepath: str,
            callback_url: str,
            callback_data: dict,
            **kwargs
    ):
        """
        授权给第三方上传, minio无回调功能，返回callback数据给前端发起回调请求
        :param filepath:
        :param callback_url: 对象存储的回调地址
        :param callback_data: 需要回传的参数
        :return:
        """
        form_data = self.post_sign_url(filepath)
        data = {
            'mode': self.mode,
            'dir': filepath,
            'host': f'{self.scheme}:{self.host}',
            'success_action_status': 200,
            'callback': {'url': callback_url, 'data': callback_data},
            # 'Content-Type': '上传时指定Content-Type',
            **form_data,
        }
        return data

    @property
    def host(self):
        return self._host_minio

    def get_key_from_url(self, url):
        """从URL中获取对象存储key"""
        return self._get_key_from_url_minio(url)

    def get_file_url(self, key, with_scheme=False):
        return self._get_file_url_minio(key, with_scheme)



#!/usr/bin/python3.6.8+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-10-20
@desc: 对阿里云oss的封装，依赖oss2
"""

import os
import json
import base64
import hmac
import datetime
import hashlib
from urllib import parse
from typing import Union, IO, AnyStr
from os import PathLike
from yzcore.extensions.storage.base import StorageManagerBase, StorageRequestError
from yzcore.extensions.storage.oss.const import *
from yzcore.extensions.storage.oss.utils import wrap_request_return_bool, wrap_request_raise_404
from yzcore.extensions.storage.schemas import OssConfig

try:
    import oss2
    from oss2 import CaseInsensitiveDict
except:
    oss2 = None


class OssManager(StorageManagerBase):

    def __init__(self, conf: OssConfig):
        super(OssManager, self).__init__(conf)
        self.internal_endpoint = conf.internal_endpoint
        self.bucket = None
        self.service = None

        self.__init()

    def __init(self, bucket_name=None):
        """初始化对象"""

        if oss2 is None:
            raise ImportError("'oss2' must be installed to use OssManager")
        if bucket_name:
            self.bucket_name = bucket_name

        self.auth = oss2.Auth(self.access_key_id, self.access_key_secret)

        # 优先内网endpoint
        if self.internal_endpoint:
            self.bucket = oss2.Bucket(self.auth, self.internal_endpoint, self.bucket_name)
        else:
            self.bucket = oss2.Bucket(self.auth, self.endpoint, self.bucket_name)

    def reload_oss(self, **kwargs):
        """重新加载oss配置"""
        self.access_key_id = kwargs.get("access_key_id")
        self.access_key_secret = kwargs.get("access_key_secret")
        self.bucket_name = kwargs.get("bucket_name")
        self.endpoint = kwargs.get("endpoint")
        self.__init()

    def create_bucket(self, bucket_name=None,
                      acl_type='private',
                      storage_type='standard',
                      redundancy_type='zrs'):
        """创建bucket，并且作为当前操作bucket"""
        permission = ACL_TYPE.get(acl_type)
        config = oss2.models.BucketCreateConfig(
            storage_class=STORAGE_CLS.get(storage_type),
            data_redundancy_type=REDUNDANCY_TYPE.get(redundancy_type)
        )
        result = self.bucket.create_bucket(permission, input=config)
        self.__init(bucket_name=bucket_name)
        return result

    def get_bucket_cors(self):
        """获取存储桶的CORS配置"""
        cors_dict = {
            'allowed_origins': [],
            'allowed_methods': [],
            'allowed_headers': [],
        }
        try:
            cors = self.bucket.get_bucket_cors()
            for rule in cors.rules:
                cors_dict['allowed_origins'] = rule.allowed_origins
                cors_dict['allowed_headers'] = rule.allowed_headers
                cors_dict['allowed_methods'] = rule.allowed_methods
        except oss2.exceptions.NoSuchCors:
            pass
        return cors_dict

    def list_buckets(self, prefix='', marker='', max_keys=100, params=None):
        """根据前缀罗列用户的Bucket。

        :param str prefix: 只罗列Bucket名为该前缀的Bucket，空串表示罗列所有的Bucket
        :param str marker: 分页标志。首次调用传空串，后续使用返回值中的next_marker
        :param int max_keys: 每次调用最多返回的Bucket数目
        :param dict params: list操作参数，传入'tag-key','tag-value'对结果进行过滤

        :return: 罗列的结果
        :rtype: oss2.models.ListBucketsResult
        """
        if not hasattr(self, 'service'):
            self.service = oss2.Service(self.auth, self.endpoint)
        return self.service.list_buckets(
            prefix=prefix, marker=marker, max_keys=max_keys, params=params)

    @wrap_request_return_bool
    def is_exist_bucket(self, **kwargs):
        """判断存储空间是否存在"""
        return self.bucket.get_bucket_info()

    def delete_bucket(self, **kwargs):
        """删除bucket"""
        try:
            resp = self.bucket.delete_bucket()
            if resp.status < 300:
                return True
            elif resp.status == 404:
                return False
        except:
            import traceback
            print(traceback.format_exc())

    def encrypt_bucket(self):
        """加密bucket"""
        # 创建Bucket加密配置，以AES256加密为例。
        rule = oss2.models.ServerSideEncryptionRule()
        rule.sse_algorithm = oss2.SERVER_SIDE_ENCRYPTION_AES256
        # 设置KMS密钥ID，加密方式为KMS可设置此项。
        # 如需使用指定的密钥加密，需输入指定的CMK ID；
        # 若使用OSS托管的CMK进行加密，此项为空。使用AES256进行加密时，此项必须为空。
        rule.kms_master_keyid = ""

        # 设置Bucket加密。
        result = self.bucket.put_bucket_encryption(rule)
        # 查看HTTP返回码。
        print('http response code:', result.status)
        return result

    def delete_encrypt_bucket(self):
        # 删除Bucket加密配置。
        result = self.bucket.delete_bucket_encryption()
        # 查看HTTP返回码。
        print('http status:', result.status)
        return result

    def get_sign_url(self, key, expire=0):
        url = self.bucket.sign_url("GET", key, expire or self.private_expire_time)
        return '//' + url.split('//', 1)[-1]

    def post_sign_url(self, key):
        pass

    def put_sign_url(self, key):
        return self.bucket.sign_url("PUT", key, self.policy_expire_time)

    def iter_objects(self, prefix='', marker='', delimiter='', max_keys=100):
        """
        遍历bucket下的文件
        :param prefix: key前缀
        :param marker:
        :param delimiter:
        :param max_keys:
        :return: dict
        """
        _result = []
        for obj in oss2.ObjectIterator(self.bucket, prefix=prefix, marker=marker, delimiter=delimiter, max_keys=max_keys):
            _result.append({
                'key': obj.key,
                'url': self.get_file_url(key=obj.key),
                'size': obj.size,
            })
        return _result

    @wrap_request_raise_404
    def download_stream(self, key, process=None):
        return self.bucket.get_object(key, process=process)

    @wrap_request_raise_404
    def download_file(self, key, local_name, process=None):
        self.bucket.get_object_to_file(key, local_name, process=process)

    def upload_file(self, filepath: Union[str, PathLike], key: str, *, num_threads=2, multipart_threshold=None):
        """
        上传文件流
        :param filepath: 文件路径
        :param key:
        :param num_threads:
        :param multipart_threshold:
        """
        headers = CaseInsensitiveDict({'Content-Type': self.parse_content_type(key)})
        result = oss2.resumable_upload(
            self.bucket, key, filepath,
            headers=headers,
            num_threads=num_threads,
            multipart_threshold=multipart_threshold,
        )
        if result.status // 100 != 2:
            raise StorageRequestError(f'oss upload error: {result.resp}')
        # 返回下载链接
        return self.get_file_url(key)

    def upload_obj(self, file_obj: Union[IO, AnyStr], key: str, **kwargs):
        """上传文件流"""
        headers = CaseInsensitiveDict({'Content-Type': self.parse_content_type(key)})
        result = self.bucket.put_object(key, file_obj, headers=headers)
        if result.status // 100 != 2:
            raise StorageRequestError(f'oss upload error: {result.resp}')
        # 返回下载链接
        return self.get_file_url(key)

    def delete_object(self, key: str):
        """删除文件"""
        self.bucket.delete_object(key)
        return True

    def get_policy(
            self,
            filepath: str,
            callback_url: str,
            callback_data: dict,
            callback_content_type: str = "application/x-www-form-urlencoded",
    ):
        """
        授权给第三方上传

        :param filepath:
        :param callback_url:
        :param callback_data: 需要回传的参数
        :param callback_content_type: 回调时的Content-Type
               "application/json"
               "application/x-www-form-urlencoded"
        :return:
        """
        params = parse.urlencode(
            dict(data=json.dumps(callback_data)))
        policy_encode = self._get_policy_encode(filepath)
        sign = self.get_signature(policy_encode)

        callback_dict = dict()
        callback_dict["callbackUrl"] = callback_url
        callback_dict["callbackBody"] = (
            "filepath=${object}&size=${size}&mime_type=${mimeType}"
            "&etag=${etag}"
            "&img_height=${imageInfo.height}&img_width=${imageInfo.width}"
            "&img_format=${imageInfo.format}&" + params
        )
        callback_dict["callbackBodyType"] = callback_content_type

        callback_param = json.dumps(callback_dict).strip().encode()
        base64_callback_body = base64.b64encode(callback_param)

        return dict(
            mode=self.mode,
            dir=filepath,
            host=f'{self.scheme}:{self.host}',
            OSSAccessKeyId=self.access_key_id,
            policy=policy_encode.decode(),
            signature=sign,
            callback=base64_callback_body.decode(),
        )

    def _get_policy_encode(self, filepath):
        expire_time = datetime.datetime.now() + datetime.timedelta(
            seconds=self.policy_expire_time
        )
        policy_dict = dict(
            expiration=expire_time.isoformat() + "Z",
            conditions=[
                ["starts-with", "$key", filepath],                    # 指定值开始
                # ["eq", "$success_action_redirect", "public-read"],  # 精确匹配
                # ["content-length-range", 1, 1024*1024*1024]         # 对象大小限制
            ],
        )
        policy = json.dumps(policy_dict).strip().encode()
        return base64.b64encode(policy)

    def get_signature(self, policy_encode):
        """
        获取签名
        :param policy_encode:
        :return:
        """
        h = hmac.new(
            self.access_key_secret.encode("utf-8"), policy_encode, hashlib.sha1
        )
        sign_result = base64.encodebytes(h.digest()).strip()
        return sign_result.decode()

    @wrap_request_raise_404
    def _set_object_headers(self, key: str, headers: dict):
        self.bucket.update_object_meta(key, headers)
        return True

    def file_exists(self, key):
        """检查文件是否存在"""
        return self.bucket.object_exists(key)

    @wrap_request_raise_404
    def get_object_meta(self, key: str):
        """获取文件基本元信息，包括该Object的ETag、Size（文件大小）、LastModified，Content-Type，并不返回其内容"""
        # meta = self.bucket.get_object_meta(key)  # get_object_meta获取到的信息有限
        meta = self.bucket.head_object(key)
        return {
            'etag': meta.etag.lower(),
            'size': meta.content_length,
            'last_modified': meta.headers['Last-Modified'],
            'content_type': meta.headers['Content-Type']
        }



#!/usr/bin/python3.7+
# -*- coding:utf-8 -*-
"""
@auth: lxm
@date: 2021/9/17
@desc: ...
"""

from sqlalchemy.orm.query import Query as _Query
from sqlalchemy.sql.selectable import Exists, Alias
from sqlalchemy.orm.mapper import Mapper
from sqlalchemy.sql.elements import Label
from yzcore.utils.fastapi_context import context


class Query(_Query):
    def __init__(self, entities, session=None):
        super().__init__(entities, session=session)
        for entity in entities:
            if type(entity) not in (Exists, Mapper, Label, Alias):
                q = self.filter_by(site_code=context.data['site_code'])
                for k, v in q.__dict__.items():
                    self.__dict__[k] = v
                break

    def join(self, *props, **kwargs):
        return super().join(*props, **kwargs).filter_by(
            site_code=context.data['site_code'])

    def outerjoin(self, *props, **kwargs):
        for prop in props:
            if not isinstance(prop, Alias):
                return super().outerjoin(*props, **kwargs).filter_by(
                    site_code=context.data['site_code'])
            else:
                return super().outerjoin(*props, **kwargs)



#!/usr/bin/python3.6+
# -*- coding:utf-8 -*-
"""
@author: Allen
@date: 2020/9/30
@desc:
"""
from hmac import compare_digest
try:
    import bcrypt
except ImportError:
    bcrypt = None


class Bcrypt(object):

    _log_rounds = 12

    @classmethod
    def generate_password_hash(cls, password: str, rounds: int = None) -> str:
        """
        Generates a password hash using bcrypt. Specifying `rounds`
        sets the log_rounds parameter of `bcrypt.gensalt()` which determines
        the complexity of the salt. 12 is the default value.

        Example usage of :class:`generate_password_hash` might look something
        like this::

            pw_hash = bcrypt.generate_password_hash('secret', 10)

        :param password: The password to be hashed.
        :param rounds: The optional number of rounds.
        """
        if rounds is None:
            rounds = cls._log_rounds
        password = password.encode()
        return bcrypt.hashpw(password, bcrypt.gensalt(rounds)).decode()

    @staticmethod
    def check_password_hash(pw_hash: str, password: str) -> bool:
        """
        Tests a password hash against a candidate password. The candidate
        password is first hashed and then subsequently compared in constant
        time to the existing hash. This will either return `True` or `False`.

        Example usage of :class:`check_password_hash` would look something
        like this::

            pw_hash = bcrypt.generate_password_hash('secret', 10)
            bcrypt.check_password_hash(pw_hash, 'secret') # returns True

        :param pw_hash: The hash to be compared against.
        :param password: The password to compare.
        """
        pw_hash = pw_hash.encode()
        password = password.encode()
        return compare_digest(bcrypt.hashpw(password, pw_hash), pw_hash)



import functools
from yzcore.exceptions import NotFoundObject
try:
    from oss2.exceptions import NotFound
except ImportError:
    NotFound = None


def wrap_request_return_bool(func):
    """查询对象是否存在"""
    @functools.wraps(func)
    def wrap_func(*args, **kwargs):
        try:
            func(*args, **kwargs)
            return True
        except NotFound:
            return False

    return wrap_func


def wrap_request_raise_404(func):
    """对象不存在时抛出404"""
    @functools.wraps(func)
    def wrap_func(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except NotFound:
            raise NotFoundObject()
    return wrap_func



import os
import shutil
from typing import Union, IO, AnyStr
from abc import ABCMeta, abstractmethod
from urllib.request import urlopen
from urllib.error import URLError
from ssl import SSLCertVerificationError

from yzcore.extensions.storage.utils import create_temp_file, get_filename, get_url_path
from yzcore.extensions.storage.const import IMAGE_FORMAT_SET, CONTENT_TYPE, DEFAULT_CONTENT_TYPE
from yzcore.extensions.storage.schemas import BaseConfig
from yzcore.exceptions import StorageRequestError
from yzcore.logger import get_logger
from yzcore.utils.decorator import cached_property


logger = get_logger(__name__)

__all__ = ['StorageManagerBase', 'logger', 'StorageRequestError']


class StorageManagerBase(metaclass=ABCMeta):

    @abstractmethod
    def __init__(self, conf: BaseConfig):
        self.mode = conf.mode
        self.access_key_id = conf.access_key_id
        self.access_key_secret = conf.access_key_secret
        self.scheme = conf.scheme.value
        self.bucket_name = conf.bucket_name
        self.endpoint = conf.endpoint
        self.image_domain = conf.image_domain
        self.asset_domain = conf.asset_domain
        self.cache_path = conf.cache_path
        self.policy_expire_time = conf.policy_expire_time  # 上传policy有效时间
        self.private_expire_time = conf.private_expire_time  # 私有桶访问链接签名有效时间

        if self.cache_path:
            self.make_dir(self.cache_path)

    @abstractmethod
    def create_bucket(self, bucket_name):
        """创建bucket"""

    @abstractmethod
    def get_bucket_cors(self):
        """
        获取CORS配置
        :return: {
            'allowed_origins': [],
            'allowed_methods': [],
            'allowed_headers': [],
        }
        """

    @abstractmethod
    def list_buckets(self):
        """查询bucket列表"""

    @abstractmethod
    def is_exist_bucket(self, bucket_name=None):
        """判断bucket是否存在"""

    @abstractmethod
    def delete_bucket(self, bucket_name=None):
        """删除bucket"""

    @abstractmethod
    def get_sign_url(self, key, expire=0):
        """生成获取文件的带签名的URL"""

    @abstractmethod
    def post_sign_url(self, key):
        """生成POST上传对象的授权信息"""

    @abstractmethod
    def put_sign_url(self, key):
        """生成PUT上传对象的带授权信息的URL"""

    @abstractmethod
    def iter_objects(self, prefix='', marker=None, delimiter=None, max_keys=100):
        """
        遍历存储桶内的文件
        目前返回的字段：
            [{
                'key': '',
                'url: '',
                'size': '',
            }]
        """

    @abstractmethod
    def get_object_meta(self, key: str):
        """获取文件基本元信息，包括该Object的ETag、Size（文件大小）、LastModified，Content-Type，并不返回其内容"""

    def update_file_headers(self, key, headers: dict):
        """更改Object的元数据信息，包括Content-Type这类标准的HTTP头部"""
        if not headers.get('Content-Type'):
            headers['Content-Type'] = self.parse_content_type(key)
        self._set_object_headers(key, headers)
        return True

    @abstractmethod
    def _set_object_headers(self, key, headers):
        """调用对象存储SDK更新object headers"""

    @abstractmethod
    def file_exists(self, key):
        """检查文件是否存在"""

    def download(self, key, local_name=None, path=None, is_stream=False, **kwargs):
        """
        下载文件
        :param key:
        :param local_name: 下载的文件在本地的路径
        :param path: 文件下载路径
        :param is_stream:
            is_stream = True:
                >>> result = self.download('readme.txt', is_stream=True)
                >>> print(result.read())
                b'hello world'
            is_stream = False:
                >>> result = self.download('readme.txt', '/tmp/cache/readme.txt')
                >>> print(result)
                '/tmp/cache/readme.txt'
        :return: 文件对象或文件下载后的本地路径
        """
        if is_stream:
            return self.download_stream(key, **kwargs)
        else:
            if not local_name:
                if path:
                    local_name = os.path.abspath(os.path.join(self.cache_path, path, get_filename(key)))
                else:
                    local_name = os.path.abspath(os.path.join(self.cache_path, key))
            self.make_dir(os.path.dirname(local_name))
            self.download_file(key, local_name)
            return local_name

    @abstractmethod
    def download_stream(self, key, **kwargs):
        """下载文件流"""

    @abstractmethod
    def download_file(self, key, local_name, **kwargs):
        """下载文件"""

    def upload(self, filepath: Union[str, os.PathLike], key: str, **kwargs):
        """上传文件"""
        return self.upload_file(filepath, key, **kwargs)

    @abstractmethod
    def upload_file(self, filepath: Union[str, os.PathLike], key: str, **kwargs):
        """上传文件"""

    @abstractmethod
    def upload_obj(self, file_obj: Union[IO, AnyStr], key: str, **kwargs):
        """上传文件流"""

    @abstractmethod
    def delete_object(self, key: str):
        """删除文件"""

    @abstractmethod
    def get_policy(
            self,
            filepath: str,
            callback_url: str,
            callback_data: dict,
            callback_content_type: str,
    ):
        """
        授权给第三方上传
        :param filepath: 对象存储中的存放路径，key的前缀
        :param callback_url: 对象存储的回调地址
        :param callback_data: 需要回传的参数
        :param callback_content_type: 回调时的Content-Type
               "application/json"
               "application/x-www-form-urlencoded"
        :return:
        """

    @cached_property
    def host(self):
        return u'//{}.{}'.format(self.bucket_name, self.endpoint)

    @cached_property
    def _host_minio(self):
        return u'//{}/{}'.format(self.endpoint, self.bucket_name)

    def get_file_url(self, key, with_scheme=False):
        """oss/obs: f'{bucket_name}.{endpoint}' 的方式拼接file_url"""
        if not any((self.image_domain, self.asset_domain)):
            resource_url = u"{}/{}".format(self.host, key)
        elif key.split('.')[-1].lower() in IMAGE_FORMAT_SET:
            resource_url = u"//{domain}/{key}".format(domain=self.image_domain, key=key)
        else:
            resource_url = u"//{domain}/{key}".format(domain=self.asset_domain, key=key)
        if with_scheme:
            resource_url = self.scheme + ':' + resource_url
        return resource_url

    def _get_file_url_minio(self, key, with_scheme=False):
        """minio/s3/azure: f'{endpoint}/{bucket_name}' 的方式拼接file_url"""
        if not any((self.image_domain, self.asset_domain)):
            resource_url = u"{}/{}".format(self._host_minio, key)
        elif key.split('.')[-1].lower() in IMAGE_FORMAT_SET:
            resource_url = u"//{domain}/{bucket}/{key}".format(
                domain=self.image_domain, bucket=self.bucket_name, key=key)
        else:
            resource_url = u"//{domain}/{bucket}/{key}".format(
                domain=self.asset_domain, bucket=self.bucket_name, key=key)
        if with_scheme:
            resource_url = self.scheme + ':' + resource_url
        return resource_url

    def delete_cache_file(self, filename):
        """删除文件缓存"""
        filepath = os.path.abspath(os.path.join(self.cache_path, filename))
        assert os.path.isfile(filepath), '非文件或文件不存在'
        os.remove(filepath)

    def search_cache_file(self, filename):
        """文件缓存搜索"""
        # 拼接绝对路径
        filepath = os.path.abspath(os.path.join(self.cache_path, filename))
        if os.path.isfile(filepath):
            return filepath
        else:
            return None

    @classmethod
    def make_dir(cls, dir_path):
        """新建目录"""
        try:
            os.makedirs(dir_path)
        except OSError:
            pass

    @classmethod
    def copy_file(cls, src, dst):
        """拷贝文件"""
        dst_dir = os.path.dirname(dst)
        cls.make_dir(dst_dir)
        shutil.copy(src, dst)

    def check(self):
        """通过上传和下载检查对象存储配置是否正确"""
        try:
            # 检查bucket是否正确
            assert self.is_exist_bucket(), f'{self.bucket_name}: No Such Bucket'
            # CORS 检查
            # assert self._cors_check(), f'{self.bucket_name}: CORS设置错误'

            # 生成一个带有随机字符串的内存文件
            temp_file, text = create_temp_file(text_length=32)

            key = f'storage_check_{text}.txt'
            logger.debug(f'file_exists: {self.file_exists(key)}')
            # 上传
            file_url = self.upload_obj(temp_file, key=key)
            logger.debug(f'upload: {file_url}')
            assert file_url, f'{self.bucket_name}: Upload Failed'
            # 加签url
            assert self._check_sign_url(key), f'{self.bucket_name}: Sign Url Error'
            # 下载
            download_file = self.download(key=f'storage_check_{text}.txt', is_stream=True)
            download_text = download_file.read().decode()
            assert download_text == text, f'{self.bucket_name}: DownloadFailed'

            # 获取文件元数据
            metadata = self.get_object_meta(key)
            logger.debug(f'get_object_meta: {metadata}')
            assert metadata, f'{self.bucket_name}: Get object metadata Failed'
            # 修改文件元数据
            assert self.update_file_headers(key, {'Content-Type': 'application/octet-stream'}), f'{self.bucket_name}: Update object metadata Failed'
            logger.debug(f'update_file_headers: {self.get_object_meta(key)}')

            # 遍历文件
            objects = self.iter_objects(key)
            logger.debug(f'iter_objects: {objects}')
            assert objects, f'{self.bucket_name} iter objects Failed'

            # 删除文件
            self.delete_object(key)
            logger.debug(f'file_exists: {self.file_exists(key)}')
            assert not self.file_exists(key), f'{self.bucket_name} delete object Failed'

            # 生成post policy
            policy = self.get_policy(filepath='upload_policy/', callback_url='https://hub.realibox.com/api/hub/v1/test', callback_data={'a':'b'})
            logger.debug(f'get_policy: {policy}')
            assert isinstance(policy, dict), f'{self.bucket_name}: Get policy Failed'

            return True
        except AssertionError as e:
            raise StorageRequestError(e)

    def _cors_check(self):
        """检查存储桶的CORS配置是否设置正确"""
        allowed_methods = {'GET', 'PUT', 'POST', 'DELETE', 'HEAD'}
        cors_dict = self.get_bucket_cors()
        logger.debug(f'_cors_check: {cors_dict}')
        if set(cors_dict['allowed_methods']) != allowed_methods:
            raise StorageRequestError(f'{self.bucket_name}: CORS设置错误')
        if cors_dict['allowed_headers'] != ['*']:
            raise StorageRequestError(f'{self.bucket_name}: CORS设置错误')
        if cors_dict['allowed_origins'] != ['*']:
            raise StorageRequestError(f'{self.bucket_name}: CORS设置错误')
        return True

    def _check_sign_url(self, key):
        """判断加签url是否可以正常打开，并且配置了https"""
        try:
            sign_url = self.get_sign_url(key=key, expire=600)
            resp = urlopen(self.scheme + ':' + sign_url)
            assert resp.status < 300, f'{self.bucket_name}: Sign Url Error, {sign_url}'
        except URLError as e:
            if isinstance(e.reason, SSLCertVerificationError):
                raise StorageRequestError(f'{self.bucket_name}: 未开启https')
            raise StorageRequestError(f'{self.bucket_name}: Sign Url Error')
        return True

    @staticmethod
    def parse_content_type(filename):
        ext = filename.split('.')[-1].lower()
        return CONTENT_TYPE.get(ext, DEFAULT_CONTENT_TYPE)

    def get_key_from_url(self, url):
        """
        从URL中获取对象存储key
        oss/obs: 去掉最前面的 /
        """
        url_path = get_url_path(url)
        return url_path[1:]

    def _get_key_from_url_minio(self, url):
        """
        从URL中获取对象存储key
        minio/s3/azure: 去掉最前面的 f'/{bucket_name}/'
        """
        url_path = get_url_path(url)
        return url_path[len(self.bucket_name)+2:]




from datetime import datetime, timedelta
try:
    import obs
    from obs import ObsClient as _ObsClient, const, util, client
except:
    obs = None


class ObsClient(_ObsClient):

    def createPostSignature(self, bucketName=None, objectKey=None, expires=300, formParams=None):
        return self._createPostSignature(bucketName, objectKey, expires, formParams, self.signature.lower() == 'v4')

    def _createPostSignature(self, bucketName=None, objectKey=None, expires=300, formParams=None, is_v4=False):
        date = datetime.utcnow()
        shortDate = date.strftime(const.SHORT_DATE_FORMAT)
        longDate = date.strftime(const.LONG_DATE_FORMAT)
        securityProvider = self._get_token()

        expires = 300 if expires is None else util.to_int(expires)
        expires = date + timedelta(seconds=expires)

        expires = expires.strftime(const.EXPIRATION_DATE_FORMAT)

        formParams = self._parse_post_params(formParams, securityProvider, is_v4,
                                             bucketName, objectKey, longDate, shortDate)
        policy = [f'{{"expiration":"{expires}",']

        # 添加callback数据
        if formParams.get('url'):
            policy.append('"callback":[')
            policy.append(f'{{"url":"{formParams.get("url", "")}"}},')
            policy.append(f'{{"body":"{formParams.get("body", "")}"}},')
            policy.append(f'{{"body-type":"{formParams.get("body-type", "")}"}},')
            policy.append('], ')

        policy.append('"conditions":[')

        matchAnyBucket = True
        matchAnyKey = True

        conditionAllowKeys = ['acl', 'bucket', 'key', 'success_action_redirect', 'redirect', 'success_action_status']

        for key, value in formParams.items():
            if key:
                key = util.to_string(key).lower()

                if key == 'bucket':
                    matchAnyBucket = False
                elif key == 'key':
                    matchAnyKey = False
                    policy.append(f'["starts-with", "$key", "{value}"],')
                    continue

                if key not in const.ALLOWED_REQUEST_HTTP_HEADER_METADATA_NAMES \
                        and not key.startswith(self.ha._get_header_prefix()) \
                        and not key.startswith(const.OBS_HEADER_PREFIX) and key not in conditionAllowKeys:
                    continue

                policy.append('{"')
                policy.append(key)
                policy.append('":"')
                policy.append(util.to_string(value))
                policy.append('"},')

        if matchAnyBucket:
            policy.append('["starts-with", "$bucket", ""],')

        if matchAnyKey:
            policy.append('["starts-with", "$key", ""],')

        policy.append(']}')

        originPolicy = ''.join(policy)

        policy = util.base64_encode(originPolicy)

        result = self._parse_post_signature_type(is_v4, securityProvider, originPolicy,
                                                 policy, formParams, shortDate, longDate)
        return client._CreatePostSignatureResponse(**result)



"""
@auth: lxm
@date：2022/04/24
@desc: ...
"""
import logging
from copy import copy

from uvicorn.logging import AccessFormatter


class CustomAccessFormatter(AccessFormatter):

    def formatMessage(self, record: logging.LogRecord) -> str:
        recordcopy = copy(record)
        (
            client_addr,
            method,
            full_path,
            http_version,
            status_code,
        ) = recordcopy.args
        recordcopy.__dict__.update(
            {
                "client_addr": client_addr,
                "status_code": status_code,
                "method": method,
                "proxies": full_path,
            }
        )
        return super(AccessFormatter, self).formatMessage(recordcopy)



import os
from yzcore.default_settings import default_setting as settings
from yzcore.logger import get_logger

logger = get_logger(__name__)


def db_connect_test(pg_engine=None, redis_conn=None, mongo_client=None, is_raise=False):
    """
    检测数据库连接
    :param pg_engine: pg连接实例
    :param redis_conn: redis连接实例
    :param mongo_client: mongodb连接实例
    :param is_raise: True 抛出异常 / False 杀掉进程引起k8s的pod异常
    """
    conn = False
    for i in range(3):
        try:
            if pg_engine:
                conn = pg_engine.connect()
                print(f'postgresql version: {".".join(map(str, conn.dialect.server_version_info))}')
                conn.close()
                break
            elif redis_conn:
                print(f'redis version: {redis_conn.info("Server")["redis_version"]}')
                break
            elif mongo_client:
                print(f'mongodb version: {mongo_client.server_info()["version"]}')
                break
        except Exception as e:
            logger.error(e)

            # 重试3次后，仍然连不上直接干掉端口和引起k8s的pod异常
            if not conn and i == 2:
                if is_raise:
                    raise e
                base_url = settings.BASE_URL
                port = base_url.split(":")[-1]
                command = "kill -9 $(netstat -nlp | grep :" + str(
                    port) + " | awk '{print $7}' | awk -F'/' '{{ print $1 }}')"
                os.system(command)
        else:
            conn = True
    return conn



"""
@auth: lxm
@date：2022/04/26
@desc: ...
"""
import logging
from yzcore.utils.fastapi_context.fastapi_context import context


class ContextFilter(logging.Filter):
    """
    This is a filter which injects contextual information into the log.

    Rather than use actual contextual information, we just use random
    data in this demo.
    """

    def filter(self, record):
        record.site_code = context.data.get('site_code', '') if context.exists() else ""
        return True



"""
@auth: lxm
@date：2022/04/26
@desc: 自定义核心包的日志格式
"""
import logging
from yzcore.logger import InitLoggerConfig, LOGGING_CONFIG

from yzcore.utils.custom_log.filter import ContextFilter


def get_logger(app_name: str, is_debug=True):
    custom_logging_config = LOGGING_CONFIG
    custom_logging_config['formatters']['custom']['format'] = '{"now": "%(created)f", "level": "%(levelname)s", "PID": "%(process)d", "pathname": "%(pathname)s", "lineno": "%(lineno)d", "message": "%(message)s", "site_code": "%(site_code)s"}'
    InitLoggerConfig(app_name, is_debug=is_debug)
    logger_name = '%s_logger' % app_name
    logger = logging.getLogger(logger_name)
    logger.addFilter(ContextFilter())
    return logger



#!/usr/bin/python3.6+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-6-30
@desc: ...
"""
from urllib.parse import urlparse
from typing import Generator
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from yzcore.db.CustomQuery import Query
from yzcore.default_settings import default_setting as settings


def get_db_engine(uri=''):
    if not uri:
        uri = settings.DB_URI
    if uri is None:
        raise EnvironmentError('需要配置"DB_URI"变量！')
    connect_args = {}
    _typ = urlparse(uri).scheme
    if _typ.startswith('sqlite'):
        # 只有SQLite才需要，其他数据库不需要。SQLite 只允许一个线程与其通信
        connect_args['check_same_thread'] = False
    else:
        connect_args['connect_timeout'] = 5
    return create_engine(uri, connect_args=connect_args)


def get_session(uri=''):
    engine = get_db_engine(uri)
    return sessionmaker(autocommit=False, autoflush=False, bind=engine)  # , query_cls=Query 私有化去掉site_code注入


def get_db() -> Generator:
    try:
        session = get_session()
        db = session()
        yield db
    finally:
        db.close()



#!/usr/bin/python3.7+
# -*- coding:utf-8 -*-
"""
@auth: lxm
@date: 2020/12/14
@desc: ...
"""
from _contextvars import ContextVar, copy_context
from collections import UserDict
from typing import Dict, Any

_request_scope_context_storage: ContextVar[Dict[Any, Any]] = ContextVar(
    "fastapi_context")


class _Context(UserDict):
    """
    A mapping with dict-like interface.
    It is using request context as a data store.
    Can be used only if context has been created in the middleware.
    """

    def __init__(self, *args: Any, **kwargs: Any):
        # not calling super on purpose
        if args or kwargs:
            raise AttributeError("Can't instantiate with attributes")

    @property
    def data(self) -> dict:  # type: ignore
        """
        Dump this to json. Object itself it not serializable.
        """
        try:
            return _request_scope_context_storage.get()
        except LookupError as e:
            raise RuntimeError(
                "You didn't use ContextMiddleware or "
                "you're trying to access `context` object "
                "outside of the request-response cycle."
            ) from e

    def exists(self) -> bool:
        return _request_scope_context_storage in copy_context()

    def copy(self) -> dict:  # type: ignore
        """
        Read only context data.
        """
        import copy

        return copy.copy(self.data)


context = _Context()



#!/usr/bin/python3.7+
# -*- coding:utf-8 -*-
"""
@auth: lxm
@date: 2021/9/17
@desc: ...
"""
from sqlalchemy import Column, String
from sqlalchemy.ext.declarative import declarative_base

_Base = declarative_base()


class Base(_Base):
    __abstract__ = True

    site_code = Column(String, index=True)
